package cz.autocont.iszr.demo.e99;

import cz.autocont.iszr.demo.AsyncDemoCall;
import cz.autocont.iszr.demo.BaseDemoCall;
import cz.autocont.iszr.demo.DemoException;
import cz.isvs.iszr.schemas.iszrasyncvypisfronty.v1.IszrAsyncVypisFrontyResponseType;
import cz.isvs.iszr.schemas.iszrasyncvypisfronty.v1.IszrAsyncVypisFrontyType;
import cz.isvs.iszr.schemas.iszrasyncvypisfronty.v1.OmezeniVypisuType;
import cz.isvs.iszr.services.iszrasyncvypisfronty.v1.IszrAsyncVypisFrontyPortType;
import cz.isvs.iszr.services.iszrasyncvypisfronty.v1.IszrAsyncVypisFrontyService;
import cz.isvs.reg.schemas.regtypy.v1.VysledekKodType;
import jakarta.xml.ws.BindingProvider;
import jakarta.xml.ws.Response;
import jakarta.xml.ws.WebServiceClient;

import javax.xml.namespace.QName;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;

/**
 * Demostrace asynchronniho volani webove sluzby E99 - IszrAsyncVypisFronty.
 * Volani je naimplentovano jako aktivni cekani. Druhou moznosti je implementovat cekani pomoci call-back
 * metody. Tento pristup je implementovany v {@link AsyncDemoCall}. 
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class DemoCallE99 extends BaseDemoCall<IszrAsyncVypisFrontyType, IszrAsyncVypisFrontyResponseType>
{
    /**
     * Vytvori data, ktera budou odeslana do webove sluzby.
     * 
     * @return Odesilana data.
     */
    @Override
    protected IszrAsyncVypisFrontyType createRequestData()
    {
        IszrAsyncVypisFrontyType result = new IszrAsyncVypisFrontyType();

        result.setZadostInfo(createZadostInfo());
        result.setZadost(createIszrAsyncVypisFrontyZadost());

        return result;
    }

    /**
     * Zavola webovou sluzbu E99.
     * 
     * @param requestData Data, ktera budou odeslana na webovou sluzbu.
     * @return Data, ktera vratila webova sluzba.
     * @throws DemoException Pokud pri volani sluzby dojde k chybe.
     */
    @Override
    protected IszrAsyncVypisFrontyResponseType callService(IszrAsyncVypisFrontyType requestData) throws DemoException
    {
        // get service port
        IszrAsyncVypisFrontyPortType port = createServicePort();

        System.out.println("Pro komunikaci budou pouzity tyto hodnoty:");
        System.out.println("Agenda=" + requestData.getZadostInfo().getAgenda());
        System.out.println("Role=" + requestData.getZadostInfo().getAgendovaRole());
        System.out.println("Ais=" + requestData.getZadostInfo().getAis());
        System.out.println("Ovm=" + requestData.getZadostInfo().getOvm());
        String target = (String) ((BindingProvider) port).getRequestContext().get(BindingProvider.ENDPOINT_ADDRESS_PROPERTY);
        System.out.println("Probiha asynchronni volani webove sluzby " + target);

        // call service
        Response<IszrAsyncVypisFrontyResponseType> handler = port.iszrAsyncVypisFrontyAsync(requestData.getZadostInfo(), requestData.getZadost());

        try
        {
            // wait for result
            IszrAsyncVypisFrontyResponseType response = handler.get();
            System.out.println("Dokonceno asynchronni volani webove sluzby " + target);
            // return result
            return response;
        }
        catch (Exception e)
        {
            throw new DemoException("Error by callign web serivce E99.", e);
        }
    }

    /**
     * Provede zpracovani dat, ktera vatila webova sluzba.
     * 
     * @param responseData Data, ktera vratila wbova sluzba.
     */
    @Override
    protected void processResponseData(IszrAsyncVypisFrontyResponseType responseData)
    {
        VysledekKodType resultCode = responseData.getOdpovedInfo().getStatus().getVysledekKod();
        String iszrId = responseData.getOdpovedInfo().getIszrZadostId();

        System.out.println("Pridelene IszrZadostId:" + iszrId);
        System.out.println("Vysledek:" + resultCode);

        // Zobrazeni poctu pozadavku ve fronte
        if (VysledekKodType.OK.equals(resultCode) && responseData.getIszrOdpoved() != null)
        {
            System.out.println("Ve fronte bylo nalezeno celkem pozadavku:" + responseData.getIszrOdpoved()
                    .getIszrAsyncVypisFrontyDataResponse().getPolozkyFronty().getPolozkaFronty().size());
        }
    }

    /**
     * Vytvori datovou cast pozadavku.
     * 
     * @return Data pozadavku
     */
    protected IszrAsyncVypisFrontyType.Zadost createIszrAsyncVypisFrontyZadost()
    {
        IszrAsyncVypisFrontyType.Zadost result = new IszrAsyncVypisFrontyType.Zadost();
        result.setIszrAsyncVypisFrontyData(new OmezeniVypisuType());
        return result;
    }

    /**
     * Vytvori proxy port na webovou sluzbu.
     * 
     * @return Proxy port
     * @throws DemoException Pokud pri vytvareni portu dojde k chybe.
     */
    private IszrAsyncVypisFrontyPortType createServicePort() throws DemoException
    {
        WebServiceClient webServiceClient = IszrAsyncVypisFrontyService.class.getAnnotation(WebServiceClient.class);
        final String TARGET_ENDPOINT_ADDRESS = MessageFormat.format(getConfig().getProperty("iszr.location"), "E99");
        final String TARGET_SERVICE_NAMESPACE = webServiceClient.targetNamespace();
        final String TARGET_SERVICE_NAME = webServiceClient.name();
        final String TARGET_WSDL_LOCATION = MessageFormat.format(getConfig().getProperty("wsdl.location"), "IszrAsyncVypisFronty.wsdl");

        // get service
        IszrAsyncVypisFrontyService service;
        try
        {
            service = new IszrAsyncVypisFrontyService(new URL(TARGET_WSDL_LOCATION), new QName(TARGET_SERVICE_NAMESPACE, TARGET_SERVICE_NAME));
        }
        catch (MalformedURLException e)
        {
            throw new DemoException("Wrong target address: " + TARGET_ENDPOINT_ADDRESS, e);
        }
        IszrAsyncVypisFrontyPortType port = service.getIszrAsyncVypisFrontyPortService();

        // setup target adress and SOAP loging
        setUpServicePort(port, "E99", TARGET_ENDPOINT_ADDRESS);
        return port;
    }

}
