package cz.autocont.iszr.demo.log;

import javax.xml.namespace.QName;
import jakarta.xml.soap.SOAPMessage;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import jakarta.xml.ws.handler.MessageContext;
import jakarta.xml.ws.handler.soap.SOAPHandler;
import jakarta.xml.ws.handler.soap.SOAPMessageContext;
import java.io.PrintStream;
import java.util.Set;

/**
 * SOAPHandler, ktery vytiskne obsah SOAP zpravy.
 * Provede tisk vsech tri zprav (request, response, fault).
 * <div style="font:italic">Projekt ISZR - (c) AutoCont CZ a.s. All rights reserved</div>
 * 
 * @author Vladimir Pach
 * @since 0.0.1
 */
public class SOAPLoggingHandler implements SOAPHandler<SOAPMessageContext>
{

    /**
     * Vystup, kam bude obsah SOAP zpravy vytisten.
     */
    private PrintStream out = System.out;

    /**
     * @return the out
     */
    public PrintStream getOut()
    {
        return out;
    }

    /**
     * @param out the out to set
     */
    public void setOut(PrintStream out)
    {
        this.out = out;
    }

    /**
     * Nedela nic.
     * 
     * @return Vzdy vrati <code>null</code>
     */
    public Set<QName> getHeaders()
    {
        return null;
    }

    /**
     * Provete tisk SOAP zpravy volanim metody {@link #logSoapMessage(SOAPMessageContext)}
     * 
     * @param smc SOAP kontext.
     * @return Vzdy vrati <code>true</code>
     */
    public boolean handleMessage(SOAPMessageContext smc)
    {
        logSoapMessage(smc);
        return true;
    }

    /**
     * Provete tisk SOAP zpravy volanim metody {@link #logSoapMessage(SOAPMessageContext)}
     * 
     * @param smc SOAP kontext.
     * @return Vzdy vrati <code>true</code>
     */
    public boolean handleFault(SOAPMessageContext smc)
    {
        logSoapMessage(smc);
        return true;
    }

    /**
     * Nedela nic.
     * 
     * @param messageContext SOAP message kontext.
     */
    public void close(MessageContext messageContext)
    {
    }

    /**
     * Provede tisk SOAP zpravy do vystupu {@link #out}.
     * 
     * @param smc SOAP kontext.
     */
    protected void logSoapMessage(SOAPMessageContext smc)
    {
        SOAPMessage message = smc.getMessage();
        try
        {
            Source source = new DOMSource(message.getSOAPPart());
            Transformer transformer = TransformerFactory.newInstance().newTransformer();
            transformer.setOutputProperty(OutputKeys.INDENT, "yes");
            StreamResult result = new StreamResult(getOut());
            transformer.transform(source, result);

            // naznaceni tisku priloh SOAP zpravy
            // for(Iterator<AttachmentPart> itx = message.getAttachments(); itx.hasNext();)
            // {
            // AttachmentPart part = itx.next();
            // byte content[] = part.getRawContentBytes();
            // getOut().print(new String(content));
            // }
        }
        catch (Exception e)
        {
            throw new IllegalStateException("Can't log SOAP message.", e);
        }
    }
}
